/************************************************************************************
*
*  File:  CS460.cpp
*
*  Purpose:  File containing all of the functions needed to create a dll that will
*	     be able to open and close and read from and write to the comm port.  The
*	     functions can also control the buffers.
*
*  Created By:  Karl Riehl
*  
*  Date: May 2, 2005
*  
*  Project:  Comm Port API - Senior Capstone 2005
*
*************************************************************************************/

#include <windows.h>

int _stdcall kr_SetRates(HANDLE com, int BaudRate, int ByteSize, DWORD parity, DWORD StopBits)
	/********************************************************************************
	*
	*  Function:  	kr_SetRates
	*
	*  Input:  	HANDLE com			-A handle to the comm port
	*
	*			int BaudRate		-Baud rate at which the communications 
	*								device operates. This member can be an 
	*								actual baud rate value, or one of the 
	*								following indexes: 
	*									110
	*									300
	*									600
	*									1200
	*									2400
	*									4800
	*									9600
	*									14400
	*									19200
	*									38400
	*									57600
	*									115200
	*									128000
	*									256000
	*		
	*			int ByteSize		-Number of bits in the bytes transmitted 
	*								and received.
	*								Must be between 5 and 8
	*
	*			DWORD parity		-Parity scheme to be used. This member 
	*								can be one of the following values. 
	*									EVENPARITY  	-  	Evan
	*									MARKPARITY	-	Mark
	*									NOPARITY	-	No Parity
	*									ODDPARITY	-	Odd
	*									SPACEPARITY	-	Space
	*
	*			DWORD StopBits		-Number of stop bits to be used. This 
	*								member can be one of the following values. 
	*									ONESTOPBIT	-	1 stop bit
	*									ONESTOPBITS	-	1.5 stop bits
	*									TWOSTOPBITS	-	2 stop bits
	*
	*  Output:	Returns 0 if the getting or setting of the DCB structure fails.
	*		Otherwise, the function returns 1;
	*
	*  Purpose:	The function gets the current communication rates, applies the 
	*		inputed rates, and then sets the rates.
	*
	*********************************************************************************/
{
		
	DCB dcb;	// Structure needed for the different communication rates

	// Fills the DCB structure
	if (!GetCommState(com, &dcb))
		return 0;

	// Sets the fields in the structure
	dcb.BaudRate = BaudRate;
	dcb.ByteSize = ByteSize;
	dcb.Parity = parity;
	dcb.StopBits = StopBits;

	// Sets the new communication properties
	if (!SetCommState(com, &dcb))
		return 0;

	return 1;
}

int _stdcall kr_SetTimeouts(HANDLE com, DWORD ReadInterval, int ReadMultiplier, 
			    int ReadConstant, int WriteMultiplier, int WriteConstant)
	/********************************************************************************
	*
	*  Function:  	kr_SetTimeouts
	*
	*  Input:  	HANDLE com			-A handle to the comm port
	*
	*			DWORD ReadInterval	-Maximum time allowed to elapse between 
	*								the arrival of two bytes on the 
	*								communications line, in milliseconds. 
	*								During a ReadFile operation, the time 
	*								period begins when the first byte is 
	*								received. If the interval between the 
	*								arrival of any two bytes exceeds this 
	*								amount, the ReadFile operation is 
	*								completed and any buffered data is returned. 
	*								A value of zero indicates that interval 
	*								time-outs are not used. 
	*					
	*								A value of MAXDWORD, combined with zero 
	*								values for both the ReadConstant and 
	*								ReadMultiplier members, specifies that 
	*								the read operation is to return 
	*								immediately with the bytes that have 
	*								already been received, even if no bytes 
	*								have been received.
	*							
	*			int ReadMultiplier	-Multiplier used to calculate the total 
	*								time-out period for read operations, 
	*								in milliseconds. For each read operation, 
	*								this value is multiplied by the requested 
	*								number of bytes to be read.
	*
	*			int ReadConstant	-Constant used to calculate the total 
	*								time-out period for read operations, in 
	*								milliseconds. For each read operation, 
	*								this value is added to the product of 
	*								the ReadMultiplier member and the 
	*								requested number of bytes. 
	*		
	*								A value of zero for both the ReadMultiplier 
	*								and ReadConstant members indicates that 
	*								total time-outs are not used for read 
	*								operations.
	*
	*			WriteMultiplier		-Multiplier used to calculate the total 
	*								time-out period for write operations, in 
	*								milliseconds. For each write operation, 
	*								this value is multiplied by the number 
	*								of bytes to be written. 
	*		
	*			WriteConstant		-Constant used to calculate the total 
	*								time-out period for write operations, in 
	*								milliseconds. For each write operation, 
	*								this value is added to the product of 
	*								the WriteMultiplier member and the 
	*								number of bytes to be written.
	* 
	*								A value of zero for both the WriteMultiplier 
	*								and WriteConstant members indicates that 
	*								total time-outs are not used for write 
	*								operations.
	*
	*  Output:	1 if the getting and setting of timeouts works.  Otherwise, the
	*		function returns 0.
	*
	*  Purpose:	The function sets the length in milliseconds that a function will
	*		wait before it continues on with the following code.
	*
	*********************************************************************************/
{
	COMMTIMEOUTS timeouts;

	// Fills the COMMTIMEOUTS structure
	if (!GetCommTimeouts(com, &timeouts))
		return 0;

	// Sets the fields in the structure
	timeouts.ReadIntervalTimeout = ReadInterval;
	timeouts.ReadTotalTimeoutConstant = ReadConstant;
	timeouts.ReadTotalTimeoutMultiplier = ReadMultiplier;
	timeouts.WriteTotalTimeoutConstant = WriteConstant;
	timeouts.WriteTotalTimeoutMultiplier = WriteMultiplier;

	// Sets the new timeouts
	if (!SetCommTimeouts(com, &timeouts))
		return 0;

	return 1;
}

int _stdcall kr_SetBuffer(HANDLE com, int Incoming, int Outgoing)
	/********************************************************************************
	*
	*  Function:  	kr_SetBuffer
	*
	*  Input:  	HANDLE com			-A handle to the comm port.
	*
	*			int Incomming		-The size of the read buffer
	*
	*			int Outgoing		-The size of the write buffer
	*		
	*  Output:	Returns 0 if the setting of the buffers fail.  Otherwise, the 
	*		function returns 1;
	*
	*  Purpose:	The function sets the read and write buffers.
	*
	*********************************************************************************/
{
	// Sets the new buffer size
	if (!SetupComm(com, Incoming, Outgoing))
		return 0;

	return 1;
}

int _stdcall kr_ClearOutputBuffer(HANDLE com)
	/********************************************************************************
	*
	*  Function:  	kr_ClearOutputBuffer
	*
	*  Input:  	HANDLE com			-A handle to the comm port
	*
	*  Output:	0 if the PurgeComm function fails.  Otherwise, the function will
	*		return 1.
	*
	*  Purpose:	The function clears the output buffer.
	*
	*********************************************************************************/
{
	if (!PurgeComm(com, PURGE_TXCLEAR))
		return 0;

	return 1;
}

int _stdcall kr_ClearInputBuffer(HANDLE com)
	/********************************************************************************
	*
	*  Function:  	kr_ClearInputBuffer
	*
	*  Input:  	HANDLE com			-A handle to the comm port
	*
	*  Output:	0 if the PurgeComm function fails.  Otherwise, the function will
	*		return 1.
	*
	*  Purpose:	The function clears the input buffer.
	*
	*********************************************************************************/
{
	if (!PurgeComm(com, PURGE_RXCLEAR))
		return 0;

	return 1;
}

int _stdcall kr_ClearBuffers(HANDLE com)
	/********************************************************************************
	*
	*  Function:  	kr_ClearBuffers
	*
	*  Input:  	HANDLE com			-A handle to the comm port
	*
	*  Output:	0 if there is a problem clearing both buffers.  Otherwise, a 1 is
	*		returned.
	*
	*  Purpose:	The function clears the output and input buffers.
	*
	*********************************************************************************/
{
	// Clears the output buffer
	if (!kr_ClearOutputBuffer(com))
		return 0;

	// Clears the input buffer
	if (!kr_ClearInputBuffer(com))
		return 0;
	
	return 1;
}

int _stdcall kr_ClearWrite(HANDLE com)
	/********************************************************************************
	*
	*  Function:  	kr_ClearWrite
	*
	*  Input:  	HANDLE com			-A handle to the comm port
	*
	*  Output:	0 if there is a problem with clearing the waiting write functions.
	*		Otherwise, a 1 is returned.
	*
	*  Purpose:	The function clears any waiting write functions.
	*
	*********************************************************************************/
{
	if (!PurgeComm(com, PURGE_TXABORT))
		return 0;

	return 1;
}

int _stdcall kr_ClearRead(HANDLE com)
	/********************************************************************************
	*
	*  Function:  	kr_ClearRead
	*
	*  Input:  	HANDLE com			-A handle to the comm port
	*
	*  Output:	0 if there is a problem with clearing the waiting read functions.
	*		Otherwise, a 1 is returned.
	*
	*  Purpose:	The function clears any waiting read functions.
	*
	*********************************************************************************/
{
	if (!PurgeComm(com, PURGE_RXABORT))
		return 0;

	return 1;
}

int _stdcall kr_ClearReadWrite(HANDLE com)
	/********************************************************************************
	*
	*  Function:  	kr_ClearReadWrite
	*
	*  Input:  	HANDLE com			-A handle to the comm port
	*
	*  Output:	0 if there is a problem with clearing the read and write functions.
	*		Otherwise, a 1 is returned.
	*
	*  Purpose:	The function clears any waiting read and write functions.
	*
	*********************************************************************************/
{
	// Clears the reads yet to be made
	kr_ClearRead(com);

	// Clears the writes yet to be made
	kr_ClearWrite(com);

	return 1;
}

int _stdcall kr_ClearSignal(HANDLE com)
	/********************************************************************************
	*
	*  Function:  	kr_ClearSignal
	*
	*  Input:  	HANDLE com			-A handle to the comm port
	*
	*  Output:	0 if there is a problem with clearing the read and write flags.
	*		Otherwise, a 1 is returned.
	*
	*  Purpose:	The function lets the port know that is not available to send 
	*		and receive data.
	*
	*********************************************************************************/
{
	if(!EscapeCommFunction(com, CLRDTR))
		return 0;

	if(!EscapeCommFunction(com, CLRRTS))
		return 0;

	return 1;
}

int _stdcall kr_SetSignal(HANDLE com)
	/********************************************************************************
	*
	*  Function:  	kr_SetSignal
	*
	*  Input:  	HANDLE com			-A handle to the comm port
	*
	*  Output:	0 if there is a problem with setting the read and write flags.
	*		Otherwise, a 1 is returned.
	*
	*  Purpose:	The function lets the port know that it is available to send 
	*		and receive data.
	*
	*********************************************************************************/
{
	if(!EscapeCommFunction(com, SETDTR))
		return 0;
	if(!EscapeCommFunction(com, SETRTS))
		return 0;

	return 1;
}

HANDLE _stdcall kr_OpenPort(char *port)
	/********************************************************************************
	*
	*  Function:  	kr_OpenPort
	*
	*  Input:  	HANDLE com			-A reference pointer to the comm port
	*
	*			char *port			-A pointer to the start of a string that
	*								states the port to open
	*
	*  Output:	0 if there is a problem with opening the comm port and setting
	*		the different properties.  Otherwise, a 1 is returned.
	*
	*  Purpose:	The function opens the comm port and sets the port to default
	*		settings.
	*
	*********************************************************************************/
{
	// Opens the port and sets com equal to the port handle
	HANDLE com = CreateFile(port, GENERIC_READ|GENERIC_WRITE, 0,0,OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
	
	if (com == INVALID_HANDLE_VALUE)
		return 0;
	
	// Clears the data-terminal-ready and request-to-send signal
	if (!kr_ClearSignal(com))
		return 0;	

	Sleep(250);
	
	// Sets the data-terminal-ready and request-to-send signal
	if (!kr_SetSignal(com))
		return 0;	

	Sleep(250);

	// Clears pending read and write commmands
	if (!kr_ClearReadWrite(com))
		return 0;
	
	// Clears input and output buffers
	if (!kr_ClearInputBuffer(com))
		return 0;

	Sleep(2);

	// Sets the default communication rates
	if(!kr_SetRates(com, 9600, 8, NOPARITY, ONESTOPBIT))
		return 0;

	// Sets the default read and write timeouts
	if(!kr_SetTimeouts(com, MAXWORD, 0, 0, 0, 0))
		return 0;

	// Sets the size of the buffers
	if(!kr_SetBuffer(com, 1028, 1028))
		return 0;

	return com;
}


void _stdcall kr_ClosePort(HANDLE com)
	/********************************************************************************
	*
	*  Function:  	kr_ClosePort
	*
	*  Input:  	HANDLE com			-A reference pointer to the comm port handle
	*
	*  Output:	none
	*
	*  Purpose:	The function opens the comm port and sets the port to default
	*		settings.
	*
	*********************************************************************************/
{
	CloseHandle(com);
	return;
}

int _stdcall kr_Read(HANDLE com, unsigned char* ch, int NumBytesToRead)
	/********************************************************************************
	*
	*  Function:  	kr_Read
	*
	*  Input:  	HANDLE com			-A handle to the comm port
	*
	*			unsigned char* ch	-A pointer to an unsigned char
	*
	*			int NumBytesToRead	-The number of bytes to read
	*
	*  Output:	Returns 0 if the read function fails.  Otherwise, the function
	*		returns 1.
	*
	*  Purpose:	The function reads in from the input buffer and places the first 
	*		byte at the address that is pointed to by the unsigned char pointer.
	*
	*********************************************************************************/
{
	DWORD NumBytesRead;
	if (!ReadFile(com, ch, NumBytesToRead, &NumBytesRead, 0))
		return 0;

	return (int)NumBytesRead;
}

int _stdcall kr_Write(HANDLE com, unsigned char* ch, int NumBytesToWrite)
	/********************************************************************************
	*
	*  Function:  	kr_Write
	*
	*  Input:  	HANDLE com			-A handle to the comm port
	*
	*			unsigned char* ch	-A pointer to an unsigned char
	*
	*			int NumBytesToRead	-The number of bytes to read
	*
	*  Output:	Returns 0 if the write function fails.  Otherwise, the function
	*		returns 1.
	*
	*  Purpose:	The function writes the number of bytes starting at the memory 
	*		location specified by the unsigned char pointer to the port.
	*
	*********************************************************************************/
{
	DWORD write;
    
	if (!WriteFile(com, ch, NumBytesToWrite, &write, 0))
		return 0;

	return 1;
}

int _stdcall kr_InputBufferSize(HANDLE com)
	/********************************************************************************
	*
	*  Function:  	kr_InputBufferSize
	*
	*  Input:  	HANDLE com			-A handle to the comm port
	*
	*  Output:	Returns the amount that is in the input buffer.
	*
	*  Purpose:	The function returns the number of bytes that are currently in the
	*		input buffer.
	*
	*********************************************************************************/
{
	COMSTAT cs;
	DWORD err;

	ClearCommError(com, &err, &cs);
	return cs.cbInQue;
}

int _stdcall kr_OutputBufferSize(HANDLE com)
	/********************************************************************************
	*
	*  Function:  	kr_OutputBufferSize
	*
	*  Input:  	HANDLE com			-A handle to the comm port
	*
	*  Output:	Returns the amount that is in the output buffer.
	*
	*  Purpose:	The function returns the number of bytes that are currently in the
	*		output buffer.
	*
	*********************************************************************************/
{
	COMSTAT cs;
	DWORD err;

	ClearCommError(com, &err, &cs);
	return cs.cbOutQue;
}


